//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "TDLogging.h"
#include <cstdio>
#include <cstdarg>


//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TDLogger::error(std::string message)
{
    log(LL_ERROR, message);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TDLogger::info(std::string message)
{
    log(LL_INFO, message);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
std::string TDLogger::printf(const char* format, ...)
{
    va_list argList;
    va_start(argList, format);

    const int maxFormatLength = 4000;
    char temp[maxFormatLength + 1];

#ifdef WIN32
    _vsnprintf_s(temp, maxFormatLength, format, argList);
#else
    vsprintf(temp, format, argList);
#endif

    va_end(argList);

    return temp;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void TDLogger::log(Logger::Level messageLevel, const std::string& message)
{
    switch (messageLevel)
    {
        case crchl::Logger::LL_ERROR:
        {
            ::printf("ERR: %s\n", message.c_str());
            break;
        }
        case crchl::Logger::LL_WARNING:
        {
            ::printf("w:   %s\n", message.c_str());
            break;
        }
        case crchl::Logger::LL_INFO:
        {
            ::printf("i:   %s\n", message.c_str());
            break;
        }
        case crchl::Logger::LL_DEBUG:
        {
            // No logging of debug messages in TDLogger
            // ::printf("d:   %s\n", message.c_str());
            break;
        }
    }
}


//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TDLogger* TDLogging::loggerInstance()
{
    static TDLogger sl_logger;
    return &sl_logger;
}
