//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "ExpressionCalculator.h"
#include "CRCAll.h"

#include <assert.h>
#include <math.h>
#include <algorithm>


//==================================================================================================
//
// ExpressionCalculator implementation
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ExpressionCalculator::ExpressionCalculator()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ExpressionCalculator::~ExpressionCalculator()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool ExpressionCalculator::initialize(const CRCHostMetaData& hostMetaData, const CRCInitOptions& options, CRCCalculatorMetaData* calculatorMetaData, CRCError* error)
{
    std::string init = options.initString().utf8();
    
    std::string::size_type pos = init.find("|");
    if (pos == std::string::npos)
    {
        error->setError(CRCString::fromChar("init string is not on format 'resultname|expression'"));
        return false;
    }
    
    m_resultName = init.substr(0, pos);
    m_expression = init.substr(pos + 1);
    
    if (m_resultName.size() == 0 || m_expression.size() == 0)
    {
        error->setError(CRCString::fromChar("resultname or expression is empty"));
        return false;
    }
    
    calculatorMetaData->setOutputResultName(CRCString::fromChar(m_resultName.c_str()));
    calculatorMetaData->setOutputResultType(CRC_RESTYPE_SCALAR);

    crc_size_t firstScalar = CRC_UNDEFINED_SIZE_T;

    for (size_t i = 0; i < hostMetaData.resultCount(); i++)
    {
        const CRCResultInfo& resInfo = hostMetaData.resultInfo(i);
        const std::string lowerCaseResName = CRCUtils::toLower(resInfo.name().utf8());

        if (resInfo.type() == CRC_RESTYPE_SCALAR)
        {
            firstScalar = i;
            break;
        }
    }

    if (firstScalar == CRC_UNDEFINED_SIZE_T)
    {
        error->setError(CRCString::fromChar("Could not find scalar"));
        return false;
    }
    
    if (m_expression != "sin" && m_expression != "cos" && m_expression != "sqr")
    {
        error->setError(CRCString::fromChar("Invalid expression"));
        return false;
    }
    
    calculatorMetaData->addInputResultIndex(firstScalar);
    
    return true;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void ExpressionCalculator::getCalculatorParams(CRCCalculatorParams*) const
{
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool ExpressionCalculator::setCalculatorParams(const CRCCalculatorParams&)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void ExpressionCalculator::calculateValue(const double* inputArr, size_t inputArraySize, double* outputArr, size_t outputArraySize) const
{
    // Sanity checks of input parameters
    assert(inputArr);
    assert(inputArraySize >= 1);
    assert(outputArr);
    assert(outputArraySize >= 1);
    
    double value = inputArr[0];
    
    if (m_expression == "sin")
    {
        value = sin(value);
    }
    else if (m_expression == "cos")
    {
        value = cos(value);
    }
    else if (m_expression == "sqr")
    {
        value = value*value;
    }

    // Return calculated result
    outputArr[0] = value;
}
