//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "MinimalCalculator.h"
#include "CRCAll.h"

#include <assert.h>
#include <math.h>
#include <algorithm>


//==================================================================================================
//
// MinimalCalculator implementation
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
MinimalCalculator::MinimalCalculator()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
MinimalCalculator::~MinimalCalculator()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool MinimalCalculator::initialize(const CRCHostMetaData& hostMetaData, const CRCInitOptions& options, CRCCalculatorMetaData* calculatorMetaData, CRCError* error)
{
    // We do not support instancing with a string and idString, so just go false if someone tries.
    if (options.initString().byteCount() > 0 || options.resultIdString().byteCount() > 0)
    {
        return false;
    }

    calculatorMetaData->setOutputResultName(CRCString::fromChar("Calc_VelocityPressure (MinimalCalculator)"));
    calculatorMetaData->setOutputResultType(CRC_RESTYPE_SCALAR);

    crc_size_t pressureResIdx = CRC_UNDEFINED_SIZE_T;
    crc_size_t velocityResIdx = CRC_UNDEFINED_SIZE_T;

    for (size_t i = 0; i < hostMetaData.resultCount(); i++)
    {
        const CRCResultInfo& resInfo = hostMetaData.resultInfo(i);
        const std::string lowerCaseResName = CRCUtils::toLower(resInfo.name().utf8());

        if (resInfo.type() == CRC_RESTYPE_SCALAR &&
            lowerCaseResName.find("pressure") != std::string::npos)
        {
            pressureResIdx = i;
        }

        if (resInfo.type() == CRC_RESTYPE_VECTOR &&
            lowerCaseResName.find("velocity") != std::string::npos)
        {
            velocityResIdx = i;
        }
    }

    if (pressureResIdx != CRC_UNDEFINED_SIZE_T && velocityResIdx != CRC_UNDEFINED_SIZE_T)
    {
        calculatorMetaData->addInputResultIndex(pressureResIdx);
        calculatorMetaData->addInputResultIndex(velocityResIdx);
        return true;
    }
    else
    {
        return false;
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void MinimalCalculator::getCalculatorParams(CRCCalculatorParams*) const
{
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool MinimalCalculator::setCalculatorParams(const CRCCalculatorParams&)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void MinimalCalculator::calculateValue(const double* inputArr, size_t inputArraySize, double* outputArr, size_t outputArraySize) const
{
    // Sanity checks of input parameters
    assert(inputArr);
    assert(inputArraySize >= 4);
    assert(outputArr);
    assert(outputArraySize >= 1);
    
    // Get input values
    const double pressure  = inputArr[0];
    const double velocityX = inputArr[1];
    const double velocityY = inputArr[2];
    const double velocityZ = inputArr[3];
    
    // Calculate output
    const double result = pressure * sqrt(velocityX*velocityX + velocityY*velocityY + velocityZ*velocityZ);
    
    // Return calculated result
    outputArr[0] = result;
}
