//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "ParamExampleCalculator.h"
#include "CRCAll.h"

#include <assert.h>
#include <math.h>
#include <algorithm>


//==================================================================================================
//
// ParamExampleCalculator implementations
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ParamExampleCalculator::ParamExampleCalculator(CRC_LogFunc logFunc)
:   m_logHelper(logFunc),
    m_scaleFactor(2)
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ParamExampleCalculator::~ParamExampleCalculator()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool ParamExampleCalculator::initialize(const CRCHostMetaData& hostMetaData, const CRCInitOptions& options, CRCCalculatorMetaData* calculatorMetaData, CRCError* error)
{
    // We do not support instancing with a string and idString, so just go false if someone tries.
    if (options.initString().byteCount() > 0 || options.resultIdString().byteCount() > 0)
    {
        return false;
    }
    
    calculatorMetaData->setOutputResultName(CRCString::fromChar("Calc_FirstScalarScaled (ParamExampleCalculator)"));
    calculatorMetaData->setOutputResultIdString(CRCString::fromChar("ParamResult"));
    calculatorMetaData->setOutputResultType(CRC_RESTYPE_SCALAR);

    // We'll just grab the first scalar as input
    for (size_t i = 0; i < hostMetaData.resultCount(); i++)
    {
        const CRCResultInfo& resInfo = hostMetaData.resultInfo(i);
        if (resInfo.type() == CRC_RESTYPE_SCALAR)
        {
            m_logHelper.logInfo("ParamExampleCalculator: choosing first scalar: " + std::string(resInfo.name().utf8()));
            calculatorMetaData->addInputResultIndex(i);
            return true;
        }
    }

    return false;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void ParamExampleCalculator::getCalculatorParams(CRCCalculatorParams* calculatorParams) const
{
    calculatorParams->setValue(CRCString::fromChar("scaleFactor"), m_scaleFactor);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool ParamExampleCalculator::setCalculatorParams(const CRCCalculatorParams& calculatorParams)
{
    if (calculatorParams.hasKey(CRCString::fromChar("scaleFactor")))
    {
        m_scaleFactor = calculatorParams.value(CRCString::fromChar("scaleFactor"));
        m_logHelper.logInfo("ParamExampleCalculator: setting scale factor value to " + CRCUtils::toStdString(m_scaleFactor));
        return true;
    }
    else
    {
        m_logHelper.logWarning("ParamExampleCalculator: did not get expected parameter");
        return false;
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void ParamExampleCalculator::calculateValue(const double* inputArr, size_t inputArraySize, double* outputArr, size_t outputArraySize) const
{
    // Sanity checks of input parameters
    assert(inputArr);
    assert(inputArraySize >= 1);
    assert(outputArr);
    assert(outputArraySize >= 1);
    
    const double result = m_scaleFactor*inputArr[0];

    // Return calculated result
    outputArr[0] = result;
}
