//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "ParamExampleCalculator.h"
#include "CRCAll.h"

#include <string.h>


CRC_LogFunc g_frameworkLogFunction = NULL;


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
static CRCResultCalculator* createCalculatorInstance()
{
    return new ParamExampleCalculator(g_frameworkLogFunction);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
static void destroyCalculatorInstance(CRCResultCalculator* calculatorInstanceToDestroy)
{
    if (calculatorInstanceToDestroy)
    {
        delete calculatorInstanceToDestroy;
    }
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
static void pluginExitFunc()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
extern "C" CRC_PLUGIN_EXPORT CRC_PluginExitFunc initializeResultCalculatorPlugin(const CRC_FrameworkServices* frameworkServices)
{
    CRCLogHelper logger(frameworkServices->logFunc);

    logger.logDebug("initializeResultCalculatorPlugin - ParamExamplePlugin");
    logger.logDebug("  compiler info: " + CRCUtils::compilerInfoString());

    frameworkServices->setPluginInfoFunc("name", "ParamExamplePlugin");
    frameworkServices->setPluginInfoFunc("compiler", CRCUtils::compilerInfoString().c_str());

    CRC_RegisterCalculatorParams regParams;
    regParams.pluginApiMajorVer     = CRC_API_MAJOR_VER;
    regParams.pluginApiMinorVer     = CRC_API_MINOR_VER;
    regParams.createCalculatorFunc  = createCalculatorInstance;
    regParams.destroyCalculatorFunc = destroyCalculatorInstance;

    int res = frameworkServices->registerCalculatorFunc("param_example_calculator", &regParams);
    if (res == CRC_RET_FAIL)
    {
        return NULL;
    }

    return pluginExitFunc;
}
