//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

#include "crchlCalculatorInstance.h"
#include "crchlCalculatorMetaData.h"
#include "crchlHostMetaData.h"
#include "crchlCalculatorParams.h"
#include "crchlAssert.h"

#include "CRCResultCalculator.h"


namespace crchl {

//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
CalculatorInstance::CalculatorInstance(std::string calculatorIdString, CRCResultCalculator* calculatorToWrap, CRC_DestroyCalculatorFunc destroyCalculatorFunc)
 :  m_calculatorIdString(calculatorIdString),
    m_wrappedCalculator(calculatorToWrap),
    m_destroyWrappedCalculatorFunc(destroyCalculatorFunc)
{
    CRCHL_ASSERT(m_wrappedCalculator);
    CRCHL_ASSERT(m_destroyWrappedCalculatorFunc);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
CalculatorInstance::~CalculatorInstance()
{
    CRCHL_ASSERT(m_wrappedCalculator);
    CRCHL_ASSERT(m_destroyWrappedCalculatorFunc);

    m_destroyWrappedCalculatorFunc(m_wrappedCalculator);
    m_wrappedCalculator = NULL;
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
std::string CalculatorInstance::calculatorId() const
{
    return m_calculatorIdString;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool CalculatorInstance::initialize(const HostMetaData& hostMetaData, const CRCInitOptions& options, CalculatorMetaData* calculatorMetaData, CRCError* error)
{
    return m_wrappedCalculator->initialize(hostMetaData, options, calculatorMetaData, error);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void CalculatorInstance::getCalculatorParams(CalculatorParams* calculatorParams) const
{
    m_wrappedCalculator->getCalculatorParams(calculatorParams);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool CalculatorInstance::setCalculatorParams(const CalculatorParams& calculatorParams)
{
    return m_wrappedCalculator->setCalculatorParams(calculatorParams);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void CalculatorInstance::calculateValue(const double* inputArr, size_t inputArraySize, double* outputArr, size_t outputArraySize) const
{
    return m_wrappedCalculator->calculateValue(inputArr, inputArraySize, outputArr, outputArraySize);
}

} // crchl

