//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################
#pragma once

#include "crchlBase.h"

#include "CRCPlugin.h"
#include "CRCString.h"
#include "CRCInitOptions.h"

#include <string>

class CRCResultCalculator;
class CRCError;

namespace crchl {

class CalculatorInfo;
class HostMetaData;
class CalculatorMetaData;
class CalculatorParams;


//==================================================================================================
//
// 
//
//==================================================================================================
class CalculatorInstance
{
public:
    ~CalculatorInstance();

    std::string calculatorId() const;

    bool        initialize(const HostMetaData& hostMetaData, const CRCInitOptions& options, CalculatorMetaData* calculatorMetaData, CRCError* error);

    void        getCalculatorParams(CalculatorParams* calculatorParams) const;
    bool        setCalculatorParams(const CalculatorParams& calculatorParams);
    void        calculateValue(const double* inputArr, size_t inputArraySize, double* outputArr, size_t outputArraySize) const;

private:
    CalculatorInstance(std::string calculatorIdString, CRCResultCalculator* calculatorToWrap, CRC_DestroyCalculatorFunc destroyCalculatorFunc);

private:
    std::string                 m_calculatorIdString;
    CRCResultCalculator*        m_wrappedCalculator;
    CRC_DestroyCalculatorFunc   m_destroyWrappedCalculatorFunc;

    friend class PluginManager;
};

} // crchl
