//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################
#pragma once

#include "CRCPlugin.h"

#include <string>
#include <map>
#include <vector>


namespace crchl {

class CalculatorInstance;
class DynamicLibrary;
class Logger;


//==================================================================================================
//
// 
//
//==================================================================================================
class PluginManager
{
public:
    static PluginManager*   instance();
    void                    shutdown();

    void                    setLogger(Logger* logger);

    bool                    loadPlugin(const std::string& pluginBaseFilename);

    bool                    initializeStaticPlugin(CRC_PluginInitFunc pluginInitFunc);

    size_t                  calculatorCount() const;
    std::string             calculatorId(size_t index) const;
    CalculatorInstance*     createCalculatorInstance(size_t index) const;

private:
    PluginManager();
    ~PluginManager();

    bool                    doInitializationOfPlugin(CRC_PluginInitFunc pluginInitFunc, DynamicLibrary* dynLib, std::string fullPluginFilename);
    bool                    hasCalculatorWithId(const char* calculatorIdString) const;

    int                     handleCalculatorRegistration(const char* calculatorIdString, const CRC_RegisterCalculatorParams* params);
    static int              staticHandleCalculatorRegistration(const char* calculatorIdString, const CRC_RegisterCalculatorParams* params);

    void                    handleSetPluginInfo(const char* key, const char* value);
    static void             staticHandleSetPluginInfo(const char* key, const char* value);

    static void             staticLogFunction(int level, const char* message);

private:
    struct PluginEntry
    {
        std::string                        fullPluginFilename;
        DynamicLibrary*                    dynLib;
        CRC_PluginExitFunc                 exitFunc;
        std::map<std::string, std::string> pluginInfoKeyValues;
    };

    struct CalculatorEntry
    {
        std::string                         calculatorIdString;
        CRC_CreateCalculatorFunc            createCalculatorFunc;
        CRC_DestroyCalculatorFunc           destroyCalculatorFunc;
    };

    Logger*                         m_logger;
    CRC_FrameworkServices           m_frameworkServices;

    PluginEntry*                    m_pluginBeingInitialized;
    std::vector<PluginEntry>        m_pluginEntryArr;
    std::vector<CalculatorEntry>    m_calculatorEntryArr;
};

} // crchl

