//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

//==================================================================================================
///
/// \class CRCLogHelper
///
/// Helper class to do bind to the raw log func provided by the host.
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// \typedef CRCLogHelper::RawLogFunc
/// The logger function provided by the CRC_FrameworkServices.
///
/// This is used to allow the plugin to log into whatever logging system the host app uses.
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
/// Create a helper by providing the rawLogFunc (as provided in the CRC_FrameworkServices)
//--------------------------------------------------------------------------------------------------
CRCLogHelper::CRCLogHelper(RawLogFunc rawLogFunc)
:   m_logFunc(rawLogFunc)
{
}


//--------------------------------------------------------------------------------------------------
/// Log error message
//--------------------------------------------------------------------------------------------------
void CRCLogHelper::logError(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(1, message.c_str());
    }
}


//--------------------------------------------------------------------------------------------------
/// Log warning message
//--------------------------------------------------------------------------------------------------
void CRCLogHelper::logWarning(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(2, message.c_str());
    }
}


//--------------------------------------------------------------------------------------------------
/// Log info message
//--------------------------------------------------------------------------------------------------
void CRCLogHelper::logInfo(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(3, message.c_str());
    }
}


//--------------------------------------------------------------------------------------------------
/// Log debug message
//--------------------------------------------------------------------------------------------------
void CRCLogHelper::logDebug(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(4, message.c_str());
    }
}
