//##################################################################################################
//
//  Ceetron Result Calculator Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//##################################################################################################

//==================================================================================================
///
/// \class CRCUtils
///
/// CRC utilities
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Returns a string identifying the current compiler
//--------------------------------------------------------------------------------------------------
std::string CRCUtils::compilerInfoString()
{
    std::string str = "NA";

    // MSVC
#ifdef _MSC_VER
    str = "MSVC " + toStdString(_MSC_VER);

    #ifdef _DLL
        str += ", Multithreaded DLL";
    #endif

#endif //_MSC_VER

#if defined (__APPLE__)
    str = "clang " + toStdString(__clang_major__) + "." + toStdString(__clang_minor__) + "." + toStdString(__clang_patchlevel__);
#elif defined (__GNUC__)
    str = "GCC " + toStdString(__GNUC__) + "." + toStdString(__GNUC_MINOR__) + "." + toStdString(__GNUC_PATCHLEVEL__);
#endif //__GNUC__

#if !defined(NDEBUG)
    str += ", debug";
#endif

    return str;
}


//--------------------------------------------------------------------------------------------------
/// Convert the result type to a std::string
//--------------------------------------------------------------------------------------------------
std::string CRCUtils::resultTypeToString(CRCResultType resultType)
{
    std::string name("<unknown>");
    
    switch (resultType)
    {
        case CRC_RESTYPE_SCALAR:    name = "Scalar";       break;
        case CRC_RESTYPE_VECTOR:    name = "Vector";       break;
    }
    
    return name;
}


//--------------------------------------------------------------------------------------------------
/// Convert the given value to a std::string
//--------------------------------------------------------------------------------------------------
template<typename T>
std::string CRCUtils::toStdString(const T& value)
{
    std::ostringstream oss;
    oss << value;
    return oss.str();
}

//--------------------------------------------------------------------------------------------------
/// Convert a std::string to lower case
//--------------------------------------------------------------------------------------------------
std::string CRCUtils::toLower(const std::string& str)
{
    std::string retStr;

    const size_t strLen = str.size();
    if (strLen > 0)
    {
        retStr.resize(strLen);

        for (size_t i = 0; i < strLen; ++i)
        {
            retStr[i] = static_cast<char>(std::tolower(static_cast<unsigned char>(str[i])));
        } 
    }

    return retStr;
}

